package decorator_pattern.tech;

import java.io.IOException;
import java.io.InputStream;

public class MyFileInputStreamBuffer extends InputStream {

    private final InputStream decoratedInputStream;  // 被装饰的InputStream
    private final byte[] buffer = new byte[8192];   // 缓冲区
    private int position;     // 当前缓冲区位置
    private int count;        // 缓冲区有效字节数

    // 使用InputStream而不是具体FileInputStream，提高通用性
    public MyFileInputStreamBuffer(InputStream inputStream) {
        this.decoratedInputStream = inputStream;
    }

    @Override
    public int read() throws IOException {
        // 缓冲区为空时填充
        if (position >= count) {
            if (!fillBuffer()) {
                return -1;  // 流结束
            }
        }
        // 从缓冲区读取一个字节
        return buffer[position++] & 0xFF;
    }

    @Override
    public int read(byte[] b, int off, int len) throws IOException {
        int totalRead = 0;
        while (len > 0) {
            // 缓冲区为空则填充
            if (position >= count) {
                if (!fillBuffer()) {
                    return totalRead == 0 ? -1 : totalRead;
                }
            }

            // 计算本次可读取的字节数
            int available = count - position;
            int toRead = Math.min(available, len);

            // 从缓冲区复制数据
            System.arraycopy(buffer, position, b, off, toRead);

            // 更新位置和长度
            position += toRead;
            off += toRead;
            len -= toRead;
            totalRead += toRead;
        }
        return totalRead;
    }

    private boolean fillBuffer() throws IOException {
        position = 0;
        count = decoratedInputStream.read(buffer);
        return count > 0;
    }

    @Override
    public void close() throws IOException {
        decoratedInputStream.close();  // 委托给被装饰对象
        position = count = 0;  // 重置缓冲区状态
    }

    // 可选：重写其它方法提升性能
    @Override
    public long skip(long n) throws IOException {
        if (n <= 0) return 0;

        // 先使用缓冲区中的可用数据
        int availableInBuffer = count - position;
        if (availableInBuffer >= n) {
            position += (int)n;
            return n;
        }

        // 需要跳过超过缓冲区的内容
        position = count;  // 消耗掉当前缓冲区
        return availableInBuffer + decoratedInputStream.skip(n - availableInBuffer);
    }
}
